@@ -380,22 +380,28 @@ split_and_add_empty_texts (HTMLEngine *e, gint level, GList **left, GList **righ
 /* end of helper */
 
 void
-html_engine_copy (HTMLEngine *e)
+html_engine_copy_object (HTMLEngine *e, HTMLObject **o, guint *len)
 {
 	GList *from, *to;
 
 	if (html_engine_is_selection_active (e)) {
 		html_engine_freeze (e);
 		prepare_delete_bounds (e, &from, &to, NULL, NULL);
-		e->clipboard_len = 0;
-		e->clipboard     = html_object_op_copy (HTML_OBJECT (from->data), e, from->next, to->next,
-							&e->clipboard_len);
-		printf ("copy len: %d (parent %p)\n", e->clipboard_len, e->clipboard->parent);
-		gtk_html_debug_dump_tree_simple (e->clipboard, 0);
+		*len = 0;
+		*o    = html_object_op_copy (HTML_OBJECT (from->data), e,
+				from->next, to->next, len);
+		printf ("copy len: %d (parent %p)\n", *len, (*o)->parent);
+		gtk_html_debug_dump_tree_simple (*o, 0);
 		html_engine_thaw (e);
 	}
 }
 
+void
+html_engine_copy (HTMLEngine *e)
+{
+	html_engine_copy_object(e, &e->clipboard, &e->clipboard_len);
+}
+
 struct _DeleteUndo {
 	HTMLUndoData data;
 
@@ -939,6 +945,7 @@ append_object (HTMLEngine *e, HTMLObject *o, guint len, HTMLUndoDirection dir)
 {
 	GList *left = NULL, *right = NULL;
 	HTMLObject *where;
+	gint back = 0;
 
 	html_engine_freeze (e);
 	if (html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent))) {
@@ -960,15 +967,35 @@ append_object (HTMLEngine *e, HTMLObject *o, guint len, HTMLUndoDirection dir)
 
 		html_object_split (e->cursor->object, e, NULL, e->cursor->offset, 2, &left, &right);
 		len += 2;
+		back = 1;
 
 		where = HTML_OBJECT (left->data);
 		html_clue_append_after (HTML_CLUE (where->parent), flow, where);
+
+		if (html_clueflow_is_empty (HTML_CLUEFLOW (where))) {
+			html_cursor_forward (e->cursor, e);
+			html_clue_remove (HTML_CLUE (where->parent), where);
+			html_object_destroy (where);
+			len --;
+			e->cursor->position --;
+		}
+		if (html_clueflow_is_empty (HTML_CLUEFLOW (HTML_OBJECT (flow)->next))) {
+			HTMLObject *empty;
+
+			empty = HTML_OBJECT (flow)->next;
+			html_clue_remove (HTML_CLUE (empty->parent), empty);
+			html_object_destroy (empty);
+			len --;
+			back = 0;
+		}
 	}
+
+	html_cursor_forward_n (e->cursor, e, len);
 	html_object_change_set (o, HTML_CHANGE_ALL_CALC);
 	html_engine_thaw (e);
 
-	html_cursor_forward_n (e->cursor, e, len);
 	insert_setup_undo (e, len, dir);
+	html_cursor_backward_n (e->cursor, e, back);
 }
 
 void
