@@ -49,6 +49,12 @@ HTMLObjectClass html_object_class;
 
 /* HTMLObject virtual methods.  */
 
+static void
+free_data (GQuark id, gpointer data, gpointer user_data)
+{
+	g_free (data);
+}
+
 static void
 destroy (HTMLObject *self)
 {
@@ -63,6 +69,7 @@ destroy (HTMLObject *self)
 		self->next = NULL;
 		self->prev = NULL;
 #endif
+		g_datalist_foreach (&self->object_data, free_data, NULL);
 		g_datalist_clear (&self->object_data);
 		g_free (self);
 	}
@@ -83,6 +90,7 @@ copy (HTMLObject *self,
 	dest->width = self->width;
 	dest->min_width = self->min_width;
 	dest->max_width = self->max_width;
+	dest->pref_width = self->pref_width;
 	dest->percent = self->percent;
 	dest->flags = self->flags;
 	dest->redraw_pending = self->redraw_pending;
@@ -97,10 +105,9 @@ copy (HTMLObject *self,
 static HTMLObject *
 op_copy (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, guint *len)
 {
-	gint l = html_object_get_length (self);
-
-	if ((!from || GPOINTER_TO_INT (from->data) == 0) && (!to || GPOINTER_TO_INT (to->data) == l)) {
-		*len += html_object_get_length (self);
+	if ((!from || GPOINTER_TO_INT (from->data) == 0)
+	    && (!to || GPOINTER_TO_INT (to->data) == html_object_get_length (self))) {
+		*len += html_object_get_recursive_length (self);
 
 		return html_object_dup (self);
 	} else
@@ -110,24 +117,22 @@ op_copy (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, guint *len)
 static HTMLObject *
 op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GList *right, guint *len)
 {
-	gint l = html_object_get_length (self);
-
-	if ((!from || GPOINTER_TO_INT (from->data) == 0) && (!to || GPOINTER_TO_INT (to->data) == l)) {
+	if ((!from || GPOINTER_TO_INT (from->data) == 0)
+	    && (!to || GPOINTER_TO_INT (to->data) == html_object_get_length (self))) {
 		if (!html_object_could_remove_whole (self, from, to, left, right)) {
 			HTMLObject *empty = html_engine_new_text_empty (e);
 
 			if (e->cursor->object == self)
 				e->cursor->object = empty;
 			html_clue_append_after (HTML_CLUE (self->parent), empty, self);
-			html_object_change_set (empty, HTML_CHANGE_ALL);
+			html_object_change_set (empty, HTML_CHANGE_ALL_CALC);
 			html_object_check_cut_lists (self, empty, left, right);
 		} else
 			html_object_move_cursor_before_remove (self, e);
 
-		html_object_change_set   (self,  HTML_CHANGE_ALL);
+		html_object_change_set   (self,  HTML_CHANGE_ALL_CALC);
 		html_object_remove_child (self->parent, self);
-		*len += l;
-
+		*len += html_object_get_recursive_length (self);
 
 		return self;
 	} else
@@ -135,7 +140,7 @@ op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GList *left, GL
 }
 
 static gboolean
-merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
+merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
 {
 	return FALSE;
 }
@@ -167,6 +172,8 @@ split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, gint lev
 			html_clue_prepend (HTML_CLUE (self->parent), html_engine_new_text_empty (e));
 		*left  = g_list_prepend (*left,  self->prev);
 		*right = g_list_prepend (*right, self);
+		e->cursor->object = self->prev;
+		e->cursor->offset = html_object_get_length (self->prev);
 	}
 	level--;
 
@@ -262,19 +269,19 @@ set_max_width (HTMLObject *o, HTMLPainter *painter, gint max_width)
 }
 
 static gint
-get_left_margin (HTMLObject *self, gint y)
+get_left_margin (HTMLObject *self, HTMLPainter *painter, gint y)
 {
 	return 0;
 }
 
 static gint
-get_right_margin (HTMLObject *self, gint y)
+get_right_margin (HTMLObject *self, HTMLPainter *painter, gint y)
 {
 	return MAX (self->max_width, self->width);
 }
 
 static void
-set_painter (HTMLObject *o, HTMLPainter *painter, gint max_width)
+set_painter (HTMLObject *o, HTMLPainter *painter)
 {
 }
 
@@ -298,6 +305,12 @@ get_target (HTMLObject *o)
 	return NULL;
 }
 
+static const gchar *
+get_src (HTMLObject *o)
+{
+	return NULL;
+}
+
 static HTMLAnchor *
 find_anchor (HTMLObject *o,
 	     const gchar *name,
@@ -470,6 +483,18 @@ get_length (HTMLObject *self)
 	return 1;
 }
 
+static guint
+get_line_length (HTMLObject *self, HTMLPainter *p, gint line_offset)
+{
+	return html_object_get_length (self);
+}
+
+static guint
+get_recursive_length (HTMLObject *self)
+{
+	return html_object_get_length (self);
+}
+
 static gboolean
 select_range (HTMLObject *self,
 	      HTMLEngine *engine,
@@ -613,6 +638,7 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->calc_preferred_width = calc_preferred_width;
 	klass->get_url = get_url;
 	klass->get_target = get_target;
+	klass->get_src = get_src;
 	klass->find_anchor = find_anchor;
 	klass->set_link = NULL;
 	klass->set_bg_color = set_bg_color;
@@ -633,6 +659,8 @@ html_object_class_init (HTMLObjectClass *klass,
 	klass->search = search;
 	klass->search_next = search;
 	klass->get_length = get_length;
+	klass->get_line_length = get_line_length;
+	klass->get_recursive_length = get_recursive_length;
 	klass->next = next;
 	klass->prev = prev;
 	klass->head = head;
@@ -661,6 +689,8 @@ html_object_init (HTMLObject *o,
 
 	o->width = 0;
 	o->max_width = 0;
+	o->min_width = 0;
+	o->pref_width = 0;
 	o->percent = 0;
 
 	o->flags = HTML_OBJECT_FLAG_FIXEDWIDTH; /* FIXME Why? */
@@ -712,10 +742,13 @@ html_object_op_cut (HTMLObject *self, HTMLEngine *e, GList *from, GList *to, GLi
 }
 
 gboolean
-html_object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e)
+html_object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
 {
-	if (HTML_OBJECT_TYPE (self) == HTML_OBJECT_TYPE (with)
-	    && (* HO_CLASS (self)->merge) (self, with, e)) {
+	if ((HTML_OBJECT_TYPE (self) == HTML_OBJECT_TYPE (with)
+	     /* FIXME */
+	     || (HTML_OBJECT_TYPE (self) == HTML_TYPE_TABLECELL && HTML_OBJECT_TYPE (with) == HTML_TYPE_CLUEV)
+	     || (HTML_OBJECT_TYPE (with) == HTML_TYPE_TABLECELL && HTML_OBJECT_TYPE (self) == HTML_TYPE_CLUEV))
+	    && (* HO_CLASS (self)->merge) (self, with, e, left, right)) {
 		if (with->parent)
 			html_object_remove_child (with->parent, with);
 		html_object_destroy (with);
@@ -865,42 +898,27 @@ html_object_set_max_width (HTMLObject *o, HTMLPainter *painter, gint max_width)
 }
 
 gint
-html_object_get_left_margin (HTMLObject *self, gint y)
+html_object_get_left_margin (HTMLObject *self, HTMLPainter *painter, gint y)
 {
-	return (* HO_CLASS (self)->get_left_margin) (self, y);
+	return (* HO_CLASS (self)->get_left_margin) (self, painter, y);
 }
 
 gint
-html_object_get_right_margin (HTMLObject *self, gint y)
+html_object_get_right_margin (HTMLObject *self, HTMLPainter *painter, gint y)
 {
-	return (* HO_CLASS (self)->get_right_margin) (self, y);
+	return (* HO_CLASS (self)->get_right_margin) (self, painter, y);
 }
 
-struct _hlpSetPainter {
-	HTMLPainter *painter;
-	gint max_width;
-};
-typedef struct _hlpSetPainter hlpSetPainter;
-
 static void
 set_painter_forall (HTMLObject *o, HTMLEngine *e, gpointer data)
 {
-	hlpSetPainter *sp;
-
-	sp = (hlpSetPainter *) data;
-	(* HO_CLASS (o)->set_painter) (o, sp->painter, sp->max_width);
+	(* HO_CLASS (o)->set_painter) (o, HTML_PAINTER (data));
 }
 
 void
-html_object_set_painter (HTMLObject *o, HTMLPainter *painter, gint max_width)
+html_object_set_painter (HTMLObject *o, HTMLPainter *painter)
 {
-	hlpSetPainter *sp;
-
-	sp = g_new (hlpSetPainter, 1);
-	sp->painter = painter;
-	sp->max_width = max_width;
-	html_object_forall (o, NULL, set_painter_forall, sp);
-	g_free (sp);
+	html_object_forall (o, NULL, set_painter_forall, painter);
 }
 
 void
@@ -931,6 +949,14 @@ html_object_calc_preferred_width (HTMLObject *o,
 	return o->pref_width;
 }
 
+#if 0
+gint
+html_object_get_uris (HTMLObject *o, char **link, char **target, char **src) 
+{
+	return TRUE;
+}
+#endif 
+
 const gchar *
 html_object_get_url (HTMLObject *o)
 {
@@ -943,6 +969,12 @@ html_object_get_target (HTMLObject *o)
 	return (* HO_CLASS (o)->get_target) (o);
 }
 
+const gchar *
+html_object_get_src (HTMLObject *o)
+{
+	return (* HO_CLASS (o)->get_src) (o);
+}
+
 HTMLAnchor *
 html_object_find_anchor (HTMLObject *o,
 			 const gchar *name,
@@ -1200,7 +1232,7 @@ HTMLObject *
 html_object_remove_link (HTMLObject *self,
 			 HTMLColor *color)
 {
-	return (HO_CLASS (self)->remove_link) ? (* HO_CLASS (self)->remove_link) (self, color) : NULL;
+	return (HO_CLASS (self)->set_link) ? (* HO_CLASS (self)->set_link) (self, color, NULL, NULL) : NULL;
 }
 
 guint
@@ -1209,6 +1241,18 @@ html_object_get_length (HTMLObject *self)
 	return (* HO_CLASS (self)->get_length) (self);
 }
 
+guint
+html_object_get_line_length (HTMLObject *self, HTMLPainter *p, gint line_offset)
+{
+	return (* HO_CLASS (self)->get_line_length) (self, p, line_offset);
+}
+
+guint
+html_object_get_recursive_length (HTMLObject *self)
+{
+	return (* HO_CLASS (self)->get_recursive_length) (self);
+}
+
 HTMLObject *
 html_object_next_by_type (HTMLObject *self, HTMLType t)
 {
@@ -1281,6 +1325,9 @@ html_object_cursor_forward (HTMLObject *self, HTMLCursor *cursor)
 	g_assert (self);
 	g_assert (cursor->object == self);
 
+	if (html_object_is_container (self))
+		return FALSE;
+
 	len = html_object_get_length (self);
 	if (cursor->offset < len) {
 		cursor->offset ++;
@@ -1296,6 +1343,9 @@ html_object_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
 	g_assert (self);
 	g_assert (cursor->object == self);
 
+	if (html_object_is_container (self))
+		return FALSE;
+
 	if (cursor->offset > 1 || (!html_object_prev_not_slave (self) && cursor->offset > 0)) {
 		cursor->offset --;
 		cursor->position --;
@@ -1305,7 +1355,9 @@ html_object_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
 	return FALSE;
 }
 
-/* movement on leafs */
+/*********************
+ * movement on leafs
+ */
 
 /* go up in tree so long as we can get object in neighborhood given by function next_fn */
 
@@ -1342,7 +1394,6 @@ move_object (HTMLObject *obj, HTMLObject * (*next_fn ) (HTMLObject *, HTMLObject
 	return obj;
 }
 
-
 inline HTMLObject *
 html_object_next_leaf (HTMLObject *self)
 {
@@ -1373,6 +1424,85 @@ html_object_prev_leaf_not_type (HTMLObject *self, HTMLType t)
 	return rv;
 }
 
+/* movement on cursor accepting objects */
+
+/* go up in tree so long as we can get object in neighborhood given by function next_fn */
+
+static HTMLObject *
+next_object_uptree_cursor (HTMLObject *obj, HTMLObject * (*next_fn ) (HTMLObject *))
+{
+	HTMLObject *next = NULL;
+
+	while (obj->parent && !(next = (*next_fn) (obj))) {
+		obj = obj->parent;
+		if (html_object_accepts_cursor (obj))
+			return obj;
+	}
+
+	return next;
+}
+
+/* go down in tree to leaf in way given by down_fn children */
+
+static HTMLObject *
+move_object_downtree_cursor (HTMLObject *obj, HTMLObject * (*down_fn ) (HTMLObject *))
+{
+	HTMLObject *down;
+
+	while ((down = (*down_fn) (obj))) {
+		obj = down;
+		if (html_object_accepts_cursor (obj))
+			break;
+	}
+
+	return obj;
+}
+
+static HTMLObject *
+move_object_cursor (HTMLObject *obj, gint *offset, gboolean forward,
+		    HTMLObject * (*next_fn ) (HTMLObject *), HTMLObject * (*down_fn ) (HTMLObject *))
+{
+	HTMLObject *down;
+
+	if (((*offset == 0 && forward) || (*offset && !forward)) && html_object_is_container (obj))
+		if ((down = (*down_fn) (obj))) {
+			down = move_object_downtree_cursor (down, down_fn);
+			if (down) {
+				if (html_object_is_container (down))
+					*offset = forward ? 0 : 1; /* FIXME for prev it's reversed */
+				return down;
+			}
+		}
+
+	obj = next_object_uptree_cursor (obj, next_fn);
+	if (obj) {
+		if (html_object_accepts_cursor (obj)) {
+			if (html_object_is_container (obj))
+				*offset = forward ? 1 : 0; /* FIXME for prev it's reversed */
+		} else {
+			obj = move_object_downtree_cursor (obj, down_fn);
+			if (html_object_is_container (obj))
+				*offset = forward ? 0 : 1; /* FIXME for prev it's reversed */
+		}
+	}
+
+	return obj;
+}
+
+inline HTMLObject *
+html_object_next_cursor (HTMLObject *self, gint *offset)
+{
+	return move_object_cursor (self, offset, TRUE, html_object_next_not_slave, html_object_head);
+}
+
+inline HTMLObject *
+html_object_prev_cursor (HTMLObject *self, gint *offset)
+{
+	return move_object_cursor (self, offset, FALSE, html_object_prev_not_slave, html_object_tail_not_slave);
+}
+
+/***/
+
 guint
 html_object_get_bytes (HTMLObject *self)
 {
@@ -1386,9 +1516,9 @@ html_object_get_index (HTMLObject *self, guint offset)
 }
 
 void
-html_object_set_data (HTMLObject *object, const gchar *key, gpointer data)
+html_object_set_data (HTMLObject *object, const gchar *key, const gchar *value)
 {
-	g_datalist_set_data (&object->object_data, key, data);
+	g_datalist_set_data (&object->object_data, key, g_strdup (value));
 }
 
 gpointer
@@ -1402,7 +1532,7 @@ copy_data (GQuark key_id, gpointer data, gpointer user_data)
 {
 	HTMLObject *o = HTML_OBJECT (user_data);
 
-	g_datalist_id_set_data (&o->object_data, key_id, data);
+	g_datalist_id_set_data (&o->object_data, key_id, g_strdup ((gchar *)data));
 }
 
 void
@@ -1414,7 +1544,7 @@ html_object_copy_data_from_object (HTMLObject *dst, HTMLObject *src)
 GList *
 html_object_get_bound_list (HTMLObject *self, GList *list)
 {
-	return list
+	return list && list->next
 		? (HTML_OBJECT (list->data) == self ? list->next : NULL)
 		: NULL;
 }
@@ -1543,3 +1673,79 @@ html_object_clear_word_width (HTMLObject *o)
 {
 	html_object_forall (o, NULL, clear_word_width, NULL);
 }
+
+HTMLObject *
+html_object_nth_parent (HTMLObject *self, gint n)
+{
+	while (self && n > 0) {
+		self = self->parent;
+		n --;
+	}
+
+	return self;
+}
+
+gint
+html_object_get_parent_level (HTMLObject *self)
+{
+	gint level = 0;
+
+	while (self) {
+		level ++;
+		self = self->parent;
+	}
+
+	return level;
+}
+
+GList *
+html_object_heads_list (HTMLObject *o)
+{
+	GList *list = NULL;
+
+	g_return_val_if_fail (o, NULL);
+
+	while (o) {
+		list = g_list_append (list, o);
+		o = html_object_head (o);
+	}
+
+	return list;
+}
+
+GList *
+html_object_tails_list (HTMLObject *o)
+{
+	GList *list = NULL;
+
+	g_return_val_if_fail (o, NULL);
+
+	while (o) {
+		list = g_list_append (list, o);
+		o = html_object_tail_not_slave (o);
+	}
+
+	return list;
+}
+
+static void
+merge_down (HTMLEngine *e, GList *left, GList *right)
+{
+	HTMLObject *lo;
+	HTMLObject *ro;
+
+	while (left && right) {
+		lo    = HTML_OBJECT (left->data);
+		ro    = HTML_OBJECT (right->data);
+		left  = left->next;
+		right = right->next;
+		if (!html_object_merge (lo, ro, e, left, right))
+			break;
+	}
+}
+
+void
+html_object_merge_down (HTMLObject *o, HTMLObject *w, HTMLEngine *e)
+{
+	merge_down (e, html_object_tails_list (o), html_object_heads_list (w));
+}
