@@ -30,6 +30,10 @@
 
 #include "htmlenums.h"
 
+#define HTML_CHECK_TYPE(object, otype) ((object) \
+                                        && (HTML_OBJECT (object))->klass \
+                                        && (HTML_OBJECT (object))->klass->type == otype)
+
 typedef struct _HTMLObject HTMLObject;
 typedef struct _HTMLObjectClass HTMLObjectClass;
 typedef struct _HTMLAnchor HTMLAnchor;
@@ -65,6 +69,10 @@ typedef struct _HTMLFont HTMLFont;
 typedef struct _HTMLFontManager HTMLFontManager;
 typedef struct _HTMLFontSet HTMLFontSet;
 typedef struct _HTMLForm HTMLForm;
+typedef struct _HTMLFrame HTMLFrame;
+typedef struct _HTMLFrameClass HTMLFrameClass;
+typedef struct _HTMLFrameset HTMLFrameset;
+typedef struct _HTMLFramesetClass HTMLFramesetClass;
 typedef struct _HTMLGdkPainter HTMLGdkPainter;
 typedef struct _HTMLGdkPainterClass HTMLGdkPainterClass;
 typedef struct _HTMLHidden HTMLHidden;
@@ -84,6 +92,8 @@ typedef struct _HTMLInterval HTMLInterval;
 typedef struct _HTMLLinkText HTMLLinkText;
 typedef struct _HTMLLinkTextClass HTMLLinkTextClass;
 typedef struct _HTMLList HTMLList;
+typedef struct _HTMLMap HTMLMap;
+typedef struct _HTMLMapClass HTMLMapClass;
 typedef struct _HTMLPainter HTMLPainter;
 typedef struct _HTMLPainterClass HTMLPainterClass;
 typedef struct _HTMLPoint HTMLPoint;
@@ -113,6 +123,7 @@ typedef struct _HTMLTextInputClass HTMLTextInputClass;
 typedef struct _HTMLTextSlave HTMLTextSlave;
 typedef struct _HTMLTextSlaveClass HTMLTextSlaveClass;
 typedef struct _HTMLTokenizer HTMLTokenizer;
+typedef struct _HTMLTokenizerClass HTMLTokenizerClass;
 typedef struct _HTMLUndo HTMLUndo;
 typedef struct _HTMLUndoData HTMLUndoData;
 typedef struct _HTMLUndoAction HTMLUndoAction;
